use super::API_BASE_URL;
use anyhow::{bail, Result};
use serde::Deserialize;
use std::fmt;

#[derive(Deserialize, Debug)]
pub struct Response {
    pub hits: Vec<SearchHit>,
    offset: i64,
    limit: i64,
    total_hits: i64,
}

#[derive(Deserialize, Debug)]
pub struct SearchHit {
    pub slug: String,
    pub title: String,
    pub description: String,
    pub client_side: String,
    pub server_side: String,
    pub project_type: String,
    pub downloads: i64,
    pub project_id: String,
    pub author: String,
    pub versions: Vec<String>,
    pub follows: i64,
}

impl fmt::Display for SearchHit {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{}", self.title)
    }
}

pub async fn search(query: &str, game_version: &str, loader: &str) -> Result<Response> {
    let game_version_facet = format!("versions:{game_version}");
    let loader_facet = format!("categories:{loader}");

    let facets: Vec<Vec<&str>> = vec![
        vec!["project_type:mod"],
        vec![&game_version_facet],
        vec![&loader_facet],
    ];

    let facets_json = serde_json::to_string(&facets)?;

    let url = format!(
        "{API_BASE_URL}/search?query={}&facets={}",
        query, facets_json
    );

    let response = reqwest::get(url).await?.json::<Response>().await;

    match response {
        Ok(r) => Ok(r),
        Err(err) => bail!("Error with Modrinth API: {}", err.to_string()),
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[tokio::test]
    async fn test_search() {
        let response = search("Fabric API", "1.20.4", "fabric").await;

        assert!(response.is_ok())
    }
}
